@@ -88,10 +88,84 @@ arg_completion <- function(workspace, token, funct, package = NULL, exported_onl
     }
 }
 
+
+ns_function_completion <- function(ns, token, exported_only, snippet_support) {
+    nsname <- ns$package_name
+    functs <- ns$get_symbols(want_functs = TRUE, exported_only = exported_only)
+    functs <- functs[startsWith(functs, token)]
+    if (nsname == WORKSPACE) {
+        tag <- "[workspace]"
+    } else {
+        tag <- paste0("{", nsname, "}")
+    }
+    if (isTRUE(snippet_support)) {
+        completions <- lapply(functs, function(object) {
+            list(label = object,
+                kind = CompletionItemKind$Function,
+                detail = tag,
+                insertText = paste0(object, "($0)"),
+                insertTextFormat = InsertTextFormat$Snippet,
+                data = list(
+                    type = "function",
+                    package = nsname
+            ))
+        })
+    } else {
+        completions <- lapply(functs, function(object) {
+            list(label = object,
+                kind = CompletionItemKind$Function,
+                detail = tag,
+                data = list(
+                    type = "function",
+                    package = nsname
+            ))
+        })
+    }
+    completions
+}
+
+imported_object_completion <- function(workspace, token, snippet_support) {
+    completions <- NULL
+    for (object in workspace$imported_objects$keys()) {
+        if (!startsWith(object, token)) {
+            next
+        }
+        nsname <- workspace$imported_objects$get(object)
+        ns <- workspace$get_namespace(nsname)
+        if (is.null(ns)) {
+            next
+        }
+        if (ns$exists_funct(object)) {
+            if (isTRUE(snippet_support)) {
+                item <- list(label = object,
+                    kind = CompletionItemKind$Function,
+                    detail = paste0("{", nsname, "}"),
+                    insertText = paste0(object, "($0)"),
+                    insertTextFormat = InsertTextFormat$Snippet,
+                    data = list(
+                        type = "function",
+                        package = nsname
+                ))
+            } else {
+                item <- list(label = object,
+                    kind = CompletionItemKind$Function,
+                    detail = paste0("{", nsname, "}"),
+                    data = list(
+                        type = "function",
+                        package = nsname
+                ))
+            }
+            completions <- append(completions, list(item))
+        }
+    }
+    completions
+}
+
+
 #' Complete any object in the workspace
 #' @keywords internal
 workspace_completion <- function(workspace, token,
-    package = NULL, exported_only = TRUE, capabilities = NULL) {
+    package = NULL, exported_only = TRUE, snippet_support = NULL) {
     completions <- list()
 
     if (is.null(package)) {
@@ -111,32 +185,9 @@ workspace_completion <- function(workspace, token,
             } else {
                 tag <- paste0("{", nsname, "}")
             }
-            functs <- ns$get_symbols(want_functs = TRUE, exported_only = TRUE)
-            functs <- functs[startsWith(functs, token)]
 
-            if (isTRUE(capabilities$completionItem$snippetSupport)) {
-                functs_completions <- lapply(functs, function(object) {
-                    list(label = object,
-                        kind = CompletionItemKind$Function,
-                        detail = tag,
-                        insertText = paste0(object, "($0)"),
-                        insertTextFormat = InsertTextFormat$Snippet,
-                        data = list(
-                            type = "function",
-                            package = nsname
-                    ))
-                })
-            } else {
-                functs_completions <- lapply(functs, function(object) {
-                    list(label = object,
-                        kind = CompletionItemKind$Function,
-                        detail = tag,
-                        data = list(
-                            type = "function",
-                            package = nsname
-                    ))
-                })
-            }
+            functs_completions <- ns_function_completion(ns, token,
+                exported_only = TRUE, snippet_support = snippet_support)
 
             nonfuncts <- ns$get_symbols(want_functs = FALSE, exported_only = TRUE)
             nonfuncts <- nonfuncts[startsWith(nonfuncts, token)]
@@ -169,32 +220,8 @@ workspace_completion <- function(workspace, token,
         ns <- workspace$get_namespace(package)
         if (!is.null(ns)) {
             tag <- paste0("{", package, "}")
-            functs <- ns$get_symbols(want_functs = TRUE, exported_only = FALSE)
-            functs <- functs[startsWith(functs, token)]
-
-            if (isTRUE(capabilities$completionItem$snippetSupport)) {
-                functs_completions <- lapply(functs, function(object) {
-                    list(label = object,
-                        kind = CompletionItemKind$Function,
-                        detail = tag,
-                        insertText = paste0(object, "($0)"),
-                        insertTextFormat = InsertTextFormat$Snippet,
-                        data = list(
-                            type = "function",
-                            package = package
-                    ))
-                })
-            } else {
-                functs_completions <- lapply(functs, function(object) {
-                    list(label = object,
-                        kind = CompletionItemKind$Function,
-                        detail = tag,
-                        data = list(
-                            type = "function",
-                            package = package
-                    ))
-                })
-            }
+            functs_completions <- ns_function_completion(ns, token,
+                exported_only = FALSE, snippet_support = snippet_support)
 
             nonfuncts <- ns$get_symbols(want_functs = FALSE, exported_only = FALSE)
             nonfuncts <- nonfuncts[startsWith(nonfuncts, token)]
@@ -213,6 +240,12 @@ workspace_completion <- function(workspace, token,
         }
     }
 
+    imported_object <- imported_object_completion(workspace, token, snippet_support)
+
+    completions <- c(
+        completions,
+        imported_object)
+
     completions
 }
 
@@ -230,7 +263,7 @@ scope_completion_functs_xpath <- paste(
     "equal_assign/EQ_ASSIGN[following-sibling::expr/FUNCTION]/preceding-sibling::expr[count(*)=1]/SYMBOL",
     sep = "|")
 
-scope_completion <- function(uri, workspace, token, point, capabilities = NULL) {
+scope_completion <- function(uri, workspace, token, point, snippet_support = NULL) {
     xdoc <- workspace$get_parse_data(uri)$xml_doc
     if (is.null(xdoc)) {
         return(list())
@@ -251,7 +284,7 @@ scope_completion <- function(uri, workspace, token, point, capabilities = NULL)
 
     scope_functs <- unique(xml_text(xml_find_all(enclosing_scopes, scope_completion_functs_xpath)))
     scope_functs <- scope_functs[startsWith(scope_functs, token)]
-    if (isTRUE(capabilities$completionItem$snippetSupport)) {
+    if (isTRUE(snippet_support)) {
         scope_funct_completions <- lapply(scope_functs, function(symbol) {
             list(
                 label = symbol,
@@ -286,6 +319,9 @@ completion_reply <- function(id, uri, workspace, document, point, capabilities)
                 items = list()
             )))
     }
+    snippet_support <- isTRUE(capabilities$completionItem$snippetSupport) &&
+        getOption("languageserver.snippet_support", TRUE)
+
     completions <- list()
     token_result <- document$detect_token(point, forward = FALSE)
 
@@ -299,12 +335,12 @@ completion_reply <- function(id, uri, workspace, document, point, capabilities)
                 completions,
                 constant_completion(token),
                 package_completion(token),
-                scope_completion(uri, workspace, token, point, capabilities))
+                scope_completion(uri, workspace, token, point, snippet_support))
         }
         completions <- c(
             completions,
             workspace_completion(
-                workspace, token, package, token_result$accessor == "::", capabilities))
+                workspace, token, package, token_result$accessor == "::", snippet_support))
     }
 
     call_result <- document$detect_call(point)
