@@ -144,8 +144,8 @@ find_unbalanced_bracket <- function(content, row, column, skip_empty_line = FALS
 #' @keywords internal
 enclosed_by_quotes <- function(document, point) {
     text <- document$line0(point$row)
-    col <- point$line
-    .Call("enclosed_by_quotes", PACKAGE = "languageserver", text, col)
+    col <- point$col
+    .Call("enclosed_by_quotes", PACKAGE = "languageserver", text, col - 1)
 }
 
 detect_comments <- function(content, row) {
@@ -315,14 +315,16 @@ parse_callback <- function(self, uri, version, parse_data) {
     }
 }
 
-parse_task <- function(self, uri, document) {
+parse_task <- function(self, uri, document, delay = 0) {
     version <- document$version
     content <- document$content
     create_task(
-        package_call(parse_document),
-        list(uri = uri, content = content),
+        target = package_call(parse_document),
+        args = list(uri = uri, content = content),
         callback = function(result) parse_callback(self, uri, version, result),
-        error = function(e) logger$info("parse_task:", e))
+        error = function(e) logger$info("parse_task:", e),
+        delay = delay
+    )
 }
 
 resolve_callback <- function(self, uri, version, packages) {
@@ -334,11 +336,13 @@ resolve_callback <- function(self, uri, version, packages) {
     self$workspace$update_loaded_packages()
 }
 
-resolve_task <- function(self, uri, document, packages) {
+resolve_task <- function(self, uri, document, packages, delay = 0) {
     version <- document$version
     create_task(
-        resolve_attached_packages,
-        list(pkgs = packages),
+        target = resolve_attached_packages,
+        args = list(pkgs = packages),
         callback = function(result) resolve_callback(self, uri, version, result),
-        error = function(e) logger$info("resolve_task:", e))
+        error = function(e) logger$info("resolve_task:", e),
+        delay = 0
+    )
 }
