@@ -7,7 +7,8 @@ text_document_completion  <- function(self, id, params) {
     uri <- textDocument$uri
     document <- self$workspace$documents$get(uri)
     point <- document$from_lsp_position(params$position)
-    self$deliver(completion_reply(id, uri, self$workspace, document, point))
+    self$deliver(completion_reply(id, uri, self$workspace, document, point,
+        self$ClientCapabilities$textDocument$completion))
 }
 
 #' `completionItem/resolve` request handler
@@ -149,7 +150,8 @@ text_document_document_link  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
     document <- self$workspace$documents$get(uri)
-    reply <- document_link_reply(id, uri, self$workspace, document, self$rootPath)
+    rootPath <- if (length(self$rootPath)) self$rootPath else dirname(path_from_uri(uri))
+    reply <- document_link_reply(id, uri, self$workspace, document, rootPath)
     if (is.null(reply)) {
         queue <- self$pending_replies$get(uri)[["textDocument/documentLink"]]
         queue$push(list(
